import { Injectable } from '@angular/core';
import {ApiService} from './api.service';
import {HttpClient} from '@angular/common/http';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {BehaviorSubject, Observable} from 'rxjs';
import {PatientContextService} from '../services/patient-context.service';

@Injectable({
  providedIn: 'root'
})
export class CarePlanService extends ApiService{

  constructor(httpClient: HttpClient, private patientContextService: PatientContextService) {
    super('/care-plan', httpClient);
  }

  // Inner subjects - for manually publishing updates to subscribers
  private innerCarePlan: BehaviorSubject<CarePlan | undefined> = new BehaviorSubject<CarePlan | undefined>(undefined);
  private innerActivities: BehaviorSubject<Activity[]> = new BehaviorSubject<Activity[]>([]);

  // Observables (from inner subjects) - subscribe to these to get updated info!
  public carePlan$: Observable<CarePlan | undefined> = this.innerCarePlan.asObservable();
  public activities$: Observable<Activity[]> = this.innerActivities.asObservable();

  // Refresh methods - these are the ones that actually call the server and manually publishes new data
  public refresh(): void {
    const episodeOfCareId = this.patientContextService.getCurrentEpisodeOfCare();
    if (episodeOfCareId) {
      this.getAll().subscribe((carePlan: CarePlan) => {
        this.innerCarePlan.next(carePlan);
        this.refreshActivities();
      });
      return;
    }
    this.innerCarePlan.next(undefined);
  }

  public refreshActivities(): void {
    const carePlan: CarePlan | undefined = this.innerCarePlan.getValue();
    this.innerActivities.next(carePlan?.activities || []);
  }

  public getActivityById(id: string): Activity | null {
    const activities = this.innerActivities.getValue();
    return activities.find(activity => activity.id === id) || null;
  }

  // Overriding api.service methods
  public update(resource: CarePlan, id: string, version: string): Observable<CarePlan> {
    return super.update(resource, id + '', version + '');
  }
}
