import {AfterViewInit, Component, ContentChildren, forwardRef, Input, QueryList} from '@angular/core';
import deepEqual from 'deep-equal';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {DataCardService} from '../api/data-card.service';

@Component({
  selector: 'app-checkbox',
  template: '<div></div>'
})
export class CheckboxComponent {
  @Input() label = '';
  @Input() value: any;

  constructor() { }
}

export const APP_CHECKBOX_LIST_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => CheckboxListComponent),
  multi: true
};

@Component({
  selector: 'app-checkbox-list',
  templateUrl: './checkbox-list.component.html',
  styleUrls: ['./checkbox-list.component.scss'],
  providers: [APP_CHECKBOX_LIST_CONTROL_VALUE_ACCESSOR],
})
export class CheckboxListComponent implements AfterViewInit, ControlValueAccessor {
  @Input() label = '';
  @ContentChildren(CheckboxComponent, {descendants: true}) checkboxes: QueryList<CheckboxComponent>|undefined;

  public checklistItems: {display: string, value: any, checked: boolean}[] = [];
  public innerModel: any[] = [];
  public disabled = false;

  private dirtyAble = false;
  private onChangeCallback: (newValue: any[]) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor(private dataCardService: DataCardService) {
    this.dataCardService.isDataCardClosed.subscribe(isClosed => this.setDisabledState(isClosed));
  }

  ngAfterViewInit(): void {
    if (this.checkboxes) {
      [...this.checkboxes.toArray()].forEach(x => {
        this.checklistItems.push({
          display: x.label,
          value: x.value,
          checked: (this.value || []).some(checkedItem => deepEqual(checkedItem, x.value))
        });
      });
    }
  }

  get value(): any[] {
    return this.innerModel;
  }

  set value(value: any[]) {
    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      this.checklistItems.forEach(x => {
        x.checked = (this.value || []).some(checkedItem => deepEqual(checkedItem, x.value));
      });
      if (this.dirtyAble) {
        this.onChangeCallback(value);
      }
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  writeValue(newValue: any[]): void {
    this.value = newValue;
  }

  public handleChange(item: {display: string, value: any, checked: boolean}): void {
    if (item.checked) {
      this.value = [...(this.value || []), item.value];
    } else {
      this.value = (this.value || []).filter(x => !deepEqual(item.value, x));
    }
  }

  public makeDirtyable(): void {
    this.dirtyAble = true;
  }
}

