import {Component, forwardRef, Input} from '@angular/core';
import deepEqual from 'deep-equal';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {MatCheckboxChange} from '@angular/material/checkbox';

export const APP_CHECKBOX_SINGLE_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => CheckboxSingleComponent),
  multi: true
};

@Component({
  selector: 'app-checkbox-single',
  templateUrl: './checkbox-single.component.html',
  styleUrls: ['./checkbox-single.component.scss'],
  providers: [APP_CHECKBOX_SINGLE_CONTROL_VALUE_ACCESSOR],
})
export class CheckboxSingleComponent implements ControlValueAccessor {
  @Input() name = '';
  @Input() label = '';
  @Input() displayValue = '';
  @Input() checkedValue: string | boolean = true;
  @Input() notRelevant: boolean | undefined;

  public innerModel: string | boolean = false;
  public disabled = false;

  private dirtyAble = false;

  private onChangeCallback: (newValue: any[]) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor() { }

  get value(): any {
    return this.innerModel;
  }

  set value(value: any) {

    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value);
      }
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: any): void {
    this.value = newValue;
  }

  makeDirtyable(): void {
    this.dirtyAble = true;
  }

  handleChange(event: MatCheckboxChange): void {
    this.value = event.checked ? this.checkedValue : undefined;
  }

}
