import {Component, HostListener, OnInit, ViewChild} from '@angular/core';
import {MatButton} from '@angular/material/button';
import {CprSearchEventService} from '../services/cpr-search-event.service';
import {WindowService} from '../services/window.service';

@Component({
  selector: 'app-cpr-search-fab',
  templateUrl: './cpr-search-fab.component.html',
  styleUrls: ['./cpr-search-fab.component.scss']
})
export class CprSearchFabComponent implements OnInit {

  @ViewChild('cprSearchFab') cprSearchFab?: MatButton;

  constructor(private cprSearchEventService: CprSearchEventService,
              private windowService: WindowService) {
  }

  ngOnInit(): void {
    this.cprSearchEventService.cprSearchFocused$.subscribe((focused) => {
      this.setCprSearchFabDisabled(focused); // Disable button on focus / Enable on blur
    });
  }

  public onCprSearchFabClick(): void {
    this.cprSearchEventService.emitCprSearchFabClickEvent();
  }

  @HostListener('window:scroll')
  public allowYScrollFrame(): void {
    const {x, y} = this.windowService.getXYScroll();

    const style = this.cprSearchFab?._elementRef.nativeElement.style;
    if (style) {
      style.top = `${0}px`;
      style.left = `${-x}px`;
      style.position = `relative`;
      if (y >= 72) {
        style.position = `fixed`;
        style.top = `${10}px`;
        style.left = `${331 - x}px`;
      }
    }
  }

  private setCprSearchFabDisabled(disabled: boolean): void {
    if (this.cprSearchFab) {
      this.cprSearchFab.disabled = disabled;
    }
  }
}
