import { ComponentFixture, TestBed } from '@angular/core/testing';
import { GeneratePdfButtonComponent } from './generate-pdf-button.component';
import {TemplateRef} from '@angular/core';
import {DialogService} from '../../modals/dialog/dialog.service';
import {Observable} from 'rxjs';
import {AngularMaterialModule} from '../../modules/angular-material.module';
import {PdfService} from '../pdf.service';
import {of} from 'rxjs/internal/observable/of';

describe('GeneratePdfButtonComponent', () => {
  let component: GeneratePdfButtonComponent;
  let fixture: ComponentFixture<GeneratePdfButtonComponent>;

  let dialogStub: TemplateRef<any>;
  let dialogServiceStub: DialogService;
  let dialogObservableStub: Observable<boolean>;

  let pdfServiceStub: PdfService;

  beforeEach(async () => {
    dialogServiceStub = {
      show: () => dialogObservableStub
    } as any;

    dialogStub = {
    } as any;

    pdfServiceStub = {
      generatePdf: () => of(new Blob()),
      savePdf: (blob: Blob) => {},
    } as PdfService;

    await TestBed.configureTestingModule({
      declarations: [ GeneratePdfButtonComponent ],
      imports: [AngularMaterialModule],
      providers: [
        {provide: DialogService, useValue: dialogServiceStub},
        {provide: PdfService, useValue: pdfServiceStub},
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(GeneratePdfButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('when dialog closes with "yes" option', () => {
    beforeEach(() => {
      dialogObservableStub = of(true);
    });

    it('should start generating pdf', () => {
      spyOn(pdfServiceStub, 'generatePdf').and.callThrough();
      component.openDialog();
      expect(pdfServiceStub.generatePdf).toHaveBeenCalledTimes(1);
    });
  });

  describe('when dialog closes with "no" option', () => {
    beforeEach(() => {
      dialogObservableStub = of(false);
    });

    it('should not start generating pdf', () => {
      spyOn(pdfServiceStub, 'generatePdf').and.callThrough();
      component.openDialog();
      expect(pdfServiceStub.generatePdf).toHaveBeenCalledTimes(0);
    });
  });
});
