import * as d3 from 'd3';
import {AfterViewInit, Component, ElementRef, Input, ViewChild} from '@angular/core';

export interface ChartPoint {
  value: number;
  week: number;
}


@Component({
  selector: 'app-clinical-exams-line-chart',
  templateUrl: './clinical-exams-line-chart.component.html',
  styleUrls: ['./clinical-exams-line-chart.component.scss']
})
export class ClinicalExamsLineChartComponent implements AfterViewInit {
  // @ts-ignore
  @ViewChild('container') container: ElementRef;
  private averageRange: ChartPoint[] = [
    {
      value: 15,
      week: 18
    }, {
      value: 18,
      week: 23
    }, {
      value: 24,
      week: 27
    }, {
      value: 27,
      week: 30
    }, {
      value: 31,
      week: 34
    }, {
      value: 35,
      week: 37
    }, {
      value: 38,
      week: 40
    }, {
      value: 38,
      week: 41
    }, {
      value: 38,
      week: 42
    },
  ];
  @Input() public data: ChartPoint[] = [];
  public PRESET_WIDTH = 1100;
  public PRESET_HEIGHT = 400;
  public svg!: d3.Selection<SVGSVGElement, any, HTMLElement, undefined>;
  private width = this.PRESET_WIDTH;
  private height = this.PRESET_HEIGHT;
  private margin: { left: number; right: number; top: number; bottom: number; } = {top: 10, right: 30, bottom: 30, left: 60};
  private scaleY: any;
  private scaleX: any;

  constructor() {
  }
  ngAfterViewInit(): void {
    if (!this.svg) {
      this.createChart();
      this.initializeChart();
    }
  }

  public createChart(): void {
    if (!this.container) {
      return;
    }
    this.width = this.PRESET_WIDTH - this.margin.left - this.margin.right;
    this.height = this.PRESET_HEIGHT - this.margin.top - this.margin.bottom;
    this.svg = d3.select<SVGElement, any>(this.container.nativeElement).append('svg');
    this.svg.attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom)
      .append('g')
      .attr('transform',
        'translate(' + this.margin.left + ',' + this.margin.top + ')');
    this.createFilters();
    this.drawAxis();
    // this.drawAverageRange();
  }

  private drawAxis(): void {
    this.scaleX = d3.scaleLinear()
      .domain([16, 43])
      .range([0, this.width]);
    this.scaleY = d3.scaleLinear()
      .domain([0, 45])
      .range([this.height, 0]);
    this.svg.append('g')
      .attr('transform', 'translate(0,' + this.height + ')')
      .call(d3.axisBottom(this.scaleX));
    const yAxis = d3.axisLeft(this.scaleY)
      .ticks(7)
      .tickFormat((d: any, i: number) => i % 2 === 0 ? d : null);
    const yAxisGrid = d3
      .axisLeft(this.scaleY)
      .tickSize(-this.width)
      .ticks(7);
    this.svg.append('g')
      .attr('class', 'axis')
      .call(yAxisGrid);
    this.svg.append('g')
      .attr('class', 'axis')
      .call(yAxis);

    const gridColor = '#DDDDE4';
    this.svg.selectAll('line')
      .style('stroke', gridColor);
    this.svg.selectAll('path')
      .style('stroke', gridColor);
  }

  // private drawAverageRange(): void {
  //   const line = d3.line<ChartPoint>()
  //     .curve(d3.curveBasis)
  //     .x(d => this.scaleX(d.week))
  //     .y(d => this.scaleY(d.value));
  //   this.svg.append('path')
  //     .datum(this.averageRange)
  //     .attr('fill', 'none')
  //     .attr('stroke', '#4228C4')
  //     .attr('stroke-width', 2)
  //     .attr('d', line);
  //   this.svg.append('line')
  //     .style('stroke', '#4228C4')
  //     .style('stroke-width', 2)
  //     .attr('x1', this.scaleX(17.8))
  //     .attr('y1', this.scaleY(19.9))
  //     .attr('x2', this.scaleX(18.2))
  //     .attr('y2', this.scaleY(10));
  //   this.svg.append('line')
  //     .style('stroke', '#4228C4')
  //     .style('stroke-width', 2)
  //     .attr('x1', this.scaleX(42))
  //     .attr('y1', this.scaleY(33))
  //     .attr('x2', this.scaleX(42))
  //     .attr('y2', this.scaleY(43));
  //   this.svg.append('path')
  //     .datum(this.averageRange)
  //     .attr('fill', 'none')
  //     .attr('stroke', 'rgba(168,168,210,0.3)')
  //     .attr('stroke-width', 80)
  //     .attr('d', line);
  // }

  private createFilters(): void {
    /* For the drop shadow filter... */
    const defs = this.svg.append('defs');

    const filter = defs.append('filter')
      .attr('id', 'dropshadow');

    filter.append('feDropShadow')
      .attr('stdDeviation', 0.3)
      .attr('dx', 0.3)
      .attr('dy', 0.5);

    const feMerge = filter.append('feMerge');

    feMerge.append('feMergeNode')
      .attr('in', 'offsetBlur');
    feMerge.append('feMergeNode')
      .attr('in', 'SourceGraphic');
  }

  private initializeChart(): void {
    this.width = this.PRESET_WIDTH - this.margin.left - this.margin.right;
    this.height = this.PRESET_HEIGHT - this.margin.top - this.margin.bottom;
    this.svg
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom);
    this.scaleX = d3.scaleLinear()
      .domain([16, 42])
      .range([0, this.width]);
    this.scaleY = d3.scaleLinear()
      .domain([0, 45])
      .range([this.height, 0]);

    const tooltip = d3.select('body')
      .append('div')
      .style('z-index', '500')
      .style('opacity', 0)
      .attr('class', 'tooltip')
      .style('border', 'solid')
      .style('border-width', '1px')
      .style('border-radius', '20px')
      .style('padding', '10px');

    const mouseover = () => {
      tooltip
        .style('opacity', 1);
    };
    const mousemove = (event: MouseEvent, d: ChartPoint) => {
      tooltip
        .html('<h4>Måling</h4><div>Symfysefundus mål: ' + d.value + ' cm</div><div>Gestationsalder: ' + d.week + '</div>')
        .style('top', (event.pageY - 88) + 'px')
        .style('left', (event.pageX - 78) + 'px');
    };
    const mouseleave = () => {
      tooltip
        .transition()
        .duration(200)
        .style('opacity', 0);
    };

    const circle = this.svg.selectAll('circle')
      .data(this.data);
    circle.enter().append('circle')
      .attr('cy', (d: ChartPoint) => this.scaleY(d.value))
      .attr('cx', (d: ChartPoint) => this.scaleX(d.week))
      .attr('fill', (d: ChartPoint) => this.isWithinRange(d) ? '#4228C4' : '#FF9B21')
      .attr('stroke', '#FFF')
      .attr('stroke-width', 3)
      .attr('filter', 'url(#dropshadow)')
      .attr('r', 6)
      .on('mouseover', mouseover )
      .on('mousemove', mousemove )
      .on('mouseleave', mouseleave );
  }

  // TODO: Reimplement this when we have legitimate ranges
  private isWithinRange(point: ChartPoint): boolean {
    function getRangeFromWeek(week: number): { lower: number; upper: number } {
      return {lower: week - 6, upper: week + 6};
    }
    const rangeFromWeek = getRangeFromWeek(point.week);
    return point.value >= rangeFromWeek.lower && point.value <= rangeFromWeek.upper;
  }
}
