import {Component, Input, OnInit} from '@angular/core';
import {Observable} from 'rxjs';
import {ClinicalObservationsService} from './clinical-observations.service';
import {ObservationTable} from '@nspop/gm-web-facade-api';
import {ChartPoint} from './clinical-exams-line-chart/clinical-exams-line-chart.component';
import {DataCardService} from '../../api/data-card.service';

@Component({
  selector: 'app-clinical-exams',
  templateUrl: './clinical-exams.component.html',
  styleUrls: ['./clinical-exams.component.scss', './tables.scss']
})
export class ClinicalExamsComponent implements OnInit {
  @Input() public showRelatedActivityModal: {isOpen: boolean; activityId?: string; observationType?: 'regular' | 'ultrasound'} | undefined;
  @Input() public measurementsModal: {isOpen: boolean; observationType: 'regular' | 'ultrasound' } | undefined;

  public tableObservable$: Observable<ObservationTable | undefined> = this.clinicalObservationsService.observationGroups;
  public chartPoints: ChartPoint[] = [];

  constructor(public clinicalObservationsService: ClinicalObservationsService,
              public dataCardService: DataCardService
  ) { }

  ngOnInit(): void {
    this.tableObservable$.subscribe((observationTable: ObservationTable | undefined) => {
      this.setPubisMeasurementPoints(observationTable);
    });
  }

  public createMeasurements(): void {
    if (this.measurementsModal) {
      this.measurementsModal.observationType = 'regular';
      this.measurementsModal.isOpen = true;
    }
  }

  public showActivity(activityId: string): void {
    if (this.showRelatedActivityModal) {
      this.showRelatedActivityModal.activityId = activityId;
      this.showRelatedActivityModal.isOpen = true;
      this.showRelatedActivityModal.observationType = 'regular';
    }
  }

  public setPubisMeasurementPoints(observationTable: ObservationTable | undefined): void {
    this.chartPoints = [];
    if (!observationTable?.mother?.rows) { return; }

    this.chartPoints = observationTable.mother.rows.map(tableRow => (
      {
        value: tableRow.pubismeasurement?.value,
        week: this.gestationAgeToNumber(tableRow.gestationAge),
      } as ChartPoint
    ));
  }

  private gestationAgeToNumber(gestationAge?: string): number {
    return gestationAge ? +gestationAge.split('+')[0] : -1;
  }

}
