import {Component, forwardRef} from '@angular/core';
import {DataCard} from '@nspop/gm-web-facade-api';
import deepEqual from 'deep-equal';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';

export const APP_MEASUREMENTS_SCREENINGS_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MeasurementsScreeningsComponent),
  multi: true
};

@Component({
  selector: 'app-measurements-screenings',
  templateUrl: './measurements-screenings.component.html',
  styleUrls: ['./measurements-screenings.component.scss'],
  providers: [APP_MEASUREMENTS_SCREENINGS_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class MeasurementsScreeningsComponent implements ControlValueAccessor {
  private innerModel: DataCardWrapper | undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: DataCard | undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};
  constructor() {
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: DataCard): void {
    this.value = new DataCardWrapper(newValue);
  }
  get value(): DataCardWrapper | undefined {
    return this.innerModel;
  }

  set value(value: DataCardWrapper | undefined) {
    if (!deepEqual(value?.asDataCard, this.innerModel?.asDataCard)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value?.asDataCard);
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
    }
  }
}
