import {Component, OnInit, ViewChild} from '@angular/core';
import {DataCard} from '@nspop/gm-web-facade-api';
import {finalize} from 'rxjs/operators';
import {NgForm} from '@angular/forms';
import {FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {DataCardService} from '../../api/data-card.service';
import {copy} from '../../../utils/copy';

@Component({
  selector: 'app-measurements-tab',
  templateUrl: './measurements-tab.component.html',
  styleUrls: ['./measurements-tab.component.scss'],
})
export class MeasurementsTabComponent extends FormComponent implements OnInit{
  public dataCard: DataCard | undefined;
  public disableSaveButton = false;
  public ultraSoundSaveListeners: (() => any)[] = [];
  private documentUniqueId = '';
  private version = '';

  public showRelatedActivityModal: {isOpen: boolean; activityId?: string; observationType?: 'regular' | 'ultrasound'} = {isOpen: false};
  public measurementsModal: {isOpen: boolean; observationType: 'regular' | 'ultrasound' } = {isOpen: false, observationType: 'regular'};

  @ViewChild('form', {static: true}) form: NgForm|undefined;

  constructor(private dataCardService: DataCardService) { super(); }

  ngOnInit(): void {
    this.dataCardService.selectedDataCard$.subscribe((dataCard?: DataCard) => {
      if (dataCard) {
        this.documentUniqueId = dataCard.documentUniqueId ?? '';
        this.version = dataCard.version ?? '';
        this.dataCard = copy(dataCard);
      }
    });
  }

  onUltrasoundSave(): void {
    this.ultraSoundSaveListeners.forEach( f => f());
  }

  public save(): void {
    if (this.form?.valid && this.dataCard) {
      this.disableSaveButton = true;
      this.dataCardService.update(this.dataCard, this.documentUniqueId, this.version)
        .pipe(finalize(() => this.disableSaveButton = false))
        .subscribe(() => {
          this.dataCardService.refresh();
          this.form?.form.markAsPristine();
        });
    }
  }
}
