import { TestBed } from '@angular/core/testing';

import {AppError, CprDoesNotExistError, ErrorService, PageNotFoundError} from './error.service';
import {PatientContextService} from './patient-context.service';
import {Router} from '@angular/router';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {appUrls} from '../../utils/URL_STRING_LITERALS';

describe('ErrorService', () => {
  let service: ErrorService;
  let patientContextServiceStub: PatientContextService;
  let routerStub: Router;
  let error: any;

  beforeEach(async () => {
    patientContextServiceStub = {
      setPatientExists: () => {},
    } as any;

    routerStub = {
      navigate: () => {}
    } as any;

    spyOn(console, 'error'); // Avoid logging the errors in console during test

    await TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        {provide: PatientContextService, useValue: patientContextServiceStub},
        {provide: Router, useValue: routerStub},
      ]
    }).compileComponents();
    service = TestBed.inject(ErrorService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('when an AppError is thrown', () => {
    beforeEach(() => {
      error = new AppError();
    });

    it('navigates to somethingWentWrong url', () => {
      spyOn(routerStub, 'navigate');
      service.handleError(error);
      expect(routerStub.navigate).toHaveBeenCalledTimes(1);
      expect(routerStub.navigate).toHaveBeenCalledWith([appUrls.somethingWentWrong]);
    });
  });

  describe('when a PageNotFoundError is thrown', () => {
    beforeEach(() => {
      error = new PageNotFoundError();
    });

    it('navigates to somethingWentWrong url', () => {
      spyOn(routerStub, 'navigate');
      service.handleError(error);
      expect(routerStub.navigate).toHaveBeenCalledTimes(1);
      expect(routerStub.navigate).toHaveBeenCalledWith([appUrls.somethingWentWrong]);
    });
  });

  describe('when a CprDoesNotExistError is thrown', () => {
    beforeEach(() => {
      error = new CprDoesNotExistError();
    });

    it('sets patientContext.patientExists to false', () => {
      spyOn(patientContextServiceStub, 'setPatientExists');
      service.handleError(error);
      expect(patientContextServiceStub.setPatientExists).toHaveBeenCalledTimes(1);
      expect(patientContextServiceStub.setPatientExists).toHaveBeenCalledWith(false);
    });
  });

  it('logs any error to console.error', () => {
    error = new Error();
    service.handleError(error);
    expect(console.error).toHaveBeenCalledTimes(1);
    expect(console.error).toHaveBeenCalledWith(error);
  });
});
