import {fakeAsync, TestBed} from '@angular/core/testing';

import {PatientContextService} from './patient-context.service';
import {WindowService} from './window.service';
import {skipWhile, takeLast} from 'rxjs/operators';

describe('PatientContextService', () => {
  let service: PatientContextService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        {provider: WindowService, useValue: {nativeWindow: () => ({
              document: {cookie: ''},
              localStorage: {getItem: () => null, setItem: () => undefined}}
            )}
        }
      ]
    });
    service = TestBed.inject(PatientContextService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('gets patient synchronously', () => {
    service.setCpr('0987654321');
    expect(service.getCurrentCpr()).toBe('0987654321');
  });

  it('gets episodeOfCare synchronously', () => {
    service.setEpisodeOfCare('777777');
    expect(service.getCurrentEpisodeOfCare()).toBe('777777');
  });

  it('gets xBreakTheGlassReason synchronously', () => {
    service.setXBreakTheGlassReason('værdispring');
    expect(service.getCurrentXBreakTheGlassReason()).toBe('værdispring');
  });

  it('gets hasPrivatelyMarkedData synchronously', () => {
    service.setHasPrivatelyMarkedData(true);
    expect(service.getCurrentHasPrivatelyMarkedData()).toBe(true);
  });

  describe('when episodeOfCare changes', () => {
    it('triggers observable when episodeOfCare changes', fakeAsync(async () => {
      service.setEpisodeOfCare('4');
      service.episodeOfCare$.pipe(skipWhile(val => val !== '4')).subscribe(careId => {
        expect(careId).toBe('4');
      });
    }));
  });

  describe('when xBreakTheGlassReason changes', () => {
    it('triggers observable when xBreakTheReason changes', fakeAsync(() => {
      service.setHasPrivatelyMarkedData(true);
      service.hasPrivatelyMarkedData$.pipe(takeLast(1)).subscribe((value) => {
        expect(value).toBe(true);
      });
    }));
  });

  describe('when xBreakTheGlassReason changes', () => {
    it('triggers observable when xBreakTheReason changes', fakeAsync(() => {
      service.setXBreakTheGlassReason('hestespring');
      service.xBreakTheGlassReason$.pipe(takeLast(1)).subscribe((reason) => {
        expect(reason).toBe('hestespring');
      });
    }));
  });

  describe('when patient changes', () => {
    it('triggers observable', fakeAsync(() => {
      service.setCpr('0987654321');
      service.cpr$.pipe(takeLast(1)).subscribe((cpr) => {
        expect(cpr).toBe('0987654321');
      });
    }));

    it('clears episodeOfCare', fakeAsync(() => {
      service.setCpr('0987654321');
      service.episodeOfCare$.pipe(takeLast(1)).subscribe((episode) => {
        expect(episode).toBeUndefined();
      });
    }));

    it('clears xBreakTheGlassReason', fakeAsync(() => {
      service.xBreakTheGlassReason$.pipe(takeLast(1)).subscribe((reason) => {
        expect(reason).toBeUndefined();
      });
      service.setCpr('0987654321');
    }));

    it('clears hasPrivatelyMarkedData', fakeAsync(() => {
      service.hasPrivatelyMarkedData$.pipe(takeLast(1)).subscribe((value) => {
        expect(value).toBeUndefined();
      });
      service.setCpr('0987654321');
    }));
  });
});
