import { TestBed } from '@angular/core/testing';

import { WindowService } from './window.service';
import {DocumentService} from './document.service';

describe('WindowService', () => {
  let service: WindowService;
  let expectedX: number;
  let expectedY: number;
  let compatModeStub: string;
  let documentServiceStub: DocumentService;

  beforeEach(async () => {
    expectedX = 0;
    expectedY = 0;
    compatModeStub = '';

    documentServiceStub = {
      nativeDocument: () => ({
        documentElement: {
          scrollLeft: 1,
          scrollTop: 2,
        } as any,
        body: {
          scrollLeft: 3,
          scrollTop: 4,
        } as any,
        compatMode: compatModeStub,
      }) as any,
    } as any;

    await TestBed.configureTestingModule({
      providers: [
        {provide: DocumentService, useValue: documentServiceStub}
      ],
    });
    service = TestBed.inject(WindowService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('when window.pageXOffset is undefined', () => {
    beforeEach(() => {
      service.nativeWindow().pageXOffset = undefined;
    });

    describe('and document.compatMode is empty string', () => {
      beforeEach(() => {
        compatModeStub = '';
      });

      it('returns body.scrollLeft & body.scrollTop', () => {
        expectedX = documentServiceStub.nativeDocument().body.scrollLeft;
        expectedY = documentServiceStub.nativeDocument().body.scrollTop;
        const {x, y} = service.getXYScroll();
        expect(x).toBe(expectedX);
        expect(y).toBe(expectedY);
      });

      describe('and document.compatMode is CSS1Compat', () => {
        beforeEach(() => {
          compatModeStub = 'CSS1Compat';
        });

        it('returns documentElement.scrollLeft & documentElement.scrollTop', () => {
          expectedX = documentServiceStub.nativeDocument().documentElement.scrollLeft;
          expectedY = documentServiceStub.nativeDocument().documentElement.scrollTop;
          const {x, y} = service.getXYScroll();
          expect(x).toBe(expectedX);
          expect(y).toBe(expectedY);
        });

      });
    });
  });

  describe('when window.pageXOffset is defined', () => {
    beforeEach(() => {
      service.nativeWindow().pageXOffset = 69;
      service.nativeWindow().pageYOffset = 420;
    });

    it('returns pageXOffset & pageYOffset', () => {
      expectedX = service.nativeWindow().pageXOffset;
      expectedY = service.nativeWindow().pageYOffset;
      const {x, y} = service.getXYScroll();
      expect(x).toBe(expectedX);
      expect(y).toBe(expectedY);
    });
  });
});
