import {ComponentFixture, TestBed} from '@angular/core/testing';
import {TextAreaComponent} from './text-area.component';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {FormsModule} from '@angular/forms';
import {By} from '@angular/platform-browser';
import {DataCardService} from '../api/data-card.service';
import {of} from 'rxjs/internal/observable/of';

describe('TextAreaComponent', () => {
  let component: TextAreaComponent;
  let fixture: ComponentFixture<TextAreaComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ TextAreaComponent ],
      imports: [AngularMaterialModule, NoopAnimationsModule, FormsModule],
      providers: [{provide: DataCardService, useValue: {isDataCardClosed: of(false)}}]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(TextAreaComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });
  it('detects mouse hover events', (done) => {
    fixture.whenStable().then(() => {
      const element = fixture.elementRef.nativeElement;
      element.dispatchEvent(new Event('mouseenter'));
      fixture.detectChanges();
      expect(component.hover).toBeTrue();

      element.dispatchEvent(new Event('mouseleave'));
      fixture.detectChanges();
      expect(component.hover).toBeFalse();
      done();
    });
  });

  it('registers as touched on blur', (done) => {
    let touched = false;
    component.registerOnTouched(() => touched = true);
    fixture.whenStable().then(() => {
      const element = fixture.elementRef.nativeElement;
      element.dispatchEvent(new Event('blur'));
      fixture.detectChanges();
      expect(touched).toBeTrue();
      done();
    });
  });

  it('updates inner value when ngModel is updated on the outside', () => {
    component.writeValue('tis but a test');
    expect(component.innerModel).toEqual('tis but a test');
  });

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };
    const onChange = spyOn(mock, 'onChange');
    component.registerOnChange(onChange);
    component.writeValue('whatever');
    expect(onChange).not.toHaveBeenCalled();
    fixture.debugElement.query(By.css('textarea')).triggerEventHandler('focus', {});
    component.writeValue('even more whatever');
    expect(onChange).toHaveBeenCalledWith('even more whatever');
  });

  it('becomes dirtyable when clicked', () => {
    fixture.debugElement.query(By.css('textarea')).triggerEventHandler('focus', {});
    fixture.detectChanges();
    expect((component as any).dirtyAble).toBeTrue();
  });
});
