import {VersionedSectionDirective} from './versioned-section.directive';
import {Component} from '@angular/core';
import {ComponentFixture, fakeAsync, flush, TestBed, tick} from '@angular/core/testing';
import {DocumentService} from './services/document.service';
import {WindowService} from './services/window.service';
import {By} from '@angular/platform-browser';
import {VersionedSection} from '@nspop/gm-web-facade-api';
import {FormsModule} from '@angular/forms';
import {JsonTransformerUtil} from '../utils/json-transformer.util';

@Component({
  selector: 'app-test-versioned-section-directive-component',
  template: '<input id="testContainer" [(ngModel)]="whatever" [(appVersionedSection)]="section">'
})
class TestVersionedSectionDirectiveComponent {
  public section?: VersionedSection;
  public whatever = '';
}

describe('VersionedSectionDirective', () => {
  let fixture: ComponentFixture<TestVersionedSectionDirectiveComponent>;
  let directive: VersionedSectionDirective;
  let component: TestVersionedSectionDirectiveComponent;

  beforeEach(() => {
    fixture = TestBed.configureTestingModule({
      declarations: [ TestVersionedSectionDirectiveComponent, VersionedSectionDirective ],
      imports: [FormsModule],
      providers: [
        DocumentService,
        WindowService
      ]
    })
      .createComponent(TestVersionedSectionDirectiveComponent);
    fixture.detectChanges(); // initial binding

    const domDirective = fixture.debugElement.queryAll(By.directive(VersionedSectionDirective));
    directive = domDirective[0].injector.get(VersionedSectionDirective) as VersionedSectionDirective;
    component = fixture.componentInstance;
  });
  it('should create an instance', () => {
    expect(directive).toBeTruthy();
  });

  it('shows that the user has changed but not saved if section is dirty', fakeAsync(() => {
    component.section = {dirty: true} as VersionedSection;
    fixture.detectChanges();
    tick();
    expect(directive.tooltipText).toEqual('Versionsgruppen er ændret af dig og endnu ikke gemt');
  }));

  it('hides tooltip if no author information is present', fakeAsync(() => {
    component.section = undefined;
    fixture.detectChanges();
    flush();
    expect(directive.tooltipText).toBeFalsy();
    expect(directive.tooltipClasses).toEqual('hidden');
  }));

  it('adds author infomration to the tooltip', fakeAsync(() => {
    const time = JsonTransformerUtil.transformDates('2021-01-12T13:37:01.00');
    component.section = {dirty: false, author: {name: 'John the Gun', sorId: 'healthCareOrganization'}, modifiedTime: time};
    fixture.detectChanges();
    flush();
    expect(directive.tooltipText).toContain('John the Gun');
    expect(directive.tooltipText).toContain('12.01.2021 - kl. 13.37');
  }));

});
